@@ -65,7 +65,7 @@ newMCResultBCa <- function( wdata, para, xmean, sample.names=NULL, method.names=
     stopifnot(is.matrix(para))
     stopifnot(all(dim(para)==c(2,4)))
     stopifnot(is.character(regmeth))
-    stopifnot(is.element(regmeth,c("LinReg","WLinReg","Deming","PaBa","WDeming", "PaBaLarge")))
+    stopifnot(is.element(regmeth,c("LinReg","WLinReg","Deming","PaBa","WDeming", "PaBaLarge","PBequi","TS")))
     stopifnot(is.character(cimeth))
     stopifnot(is.element(cimeth,c("bootstrap","nestedbootstrap")))
     stopifnot(is.character(bootcimeth))
@@ -151,7 +151,7 @@ newMCResultBCa <- function( wdata, para, xmean, sample.names=NULL, method.names=
 #' @param cimeth string specifying the confidence interval method
 #' @param bootcimeth string specifying the method for bootstrap confidence intervals
 #' @param error.ratio for deming regression 
-#' @param alpha value specifying the 100(1-alpha)% confidence-level
+#' @param alpha value specifying the 100(1-alpha)\% confidence-level
 #' @param glob.coef global coefficients
 #' @param rng.seed random number generator seed
 #' @param rng.kind type of the random number generator
@@ -422,6 +422,10 @@ MCResultBCa.printSummary<-function(.Object)
         regtext <- "Linear Regression"
     if(regmeth=="WLinReg") 
         regtext <- "Weighted Linear Regression"
+    if(regmeth=="PBequi") 
+        regtext <- "equivariant Passing-Bablok Regression"
+    if(regmeth=="TS") 
+        regtext <- "Theil-Sen Regression"
     if(regmeth=="Deming") 
         regtext <- "Deming Regression"
     if(regmeth=="WDeming") 
